@@ -1,9 +1,15 @@
 from contextlib import contextmanager
 
 import pytest
+
 from PIL import Image, ImageDraw
 
-from .helper import assert_image_equal, assert_image_similar, hopper
+from .helper import (
+    assert_image_equal,
+    assert_image_similar,
+    hopper,
+    mark_if_feature_version,
+)
 
 
 class TestImagingResampleVulnerability:
@@ -18,7 +24,7 @@ class TestImagingResampleVulnerability:
         ):
             with pytest.raises(MemoryError):
                 # any resampling filter will do here
-                im.im.resize((xsize, ysize), Image.BILINEAR)
+                im.im.resize((xsize, ysize), Image.Resampling.BILINEAR)
 
     def test_invalid_size(self):
         im = hopper()
@@ -81,22 +87,23 @@ class TestImagingCoreResampleAccuracy:
         for y in range(case.size[1]):
             for x in range(case.size[0]):
                 if c_px[x, y] != s_px[x, y]:
-                    message = "\nHave: \n{}\n\nExpected: \n{}".format(
-                        self.serialize_image(case), self.serialize_image(sample)
+                    message = (
+                        f"\nHave: \n{self.serialize_image(case)}\n"
+                        f"\nExpected: \n{self.serialize_image(sample)}"
                     )
                     assert s_px[x, y] == c_px[x, y], message
 
     def serialize_image(self, image):
         s_px = image.load()
         return "\n".join(
-            " ".join("{:02x}".format(s_px[x, y]) for x in range(image.size[0]))
+            " ".join(f"{s_px[x, y]:02x}" for x in range(image.size[0]))
             for y in range(image.size[1])
         )
 
     def test_reduce_box(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (8, 8), 0xE1)
-            case = case.resize((4, 4), Image.BOX)
+            case = case.resize((4, 4), Image.Resampling.BOX)
             # fmt: off
             data = ("e1 e1"
                     "e1 e1")
@@ -107,7 +114,7 @@ class TestImagingCoreResampleAccuracy:
     def test_reduce_bilinear(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (8, 8), 0xE1)
-            case = case.resize((4, 4), Image.BILINEAR)
+            case = case.resize((4, 4), Image.Resampling.BILINEAR)
             # fmt: off
             data = ("e1 c9"
                     "c9 b7")
@@ -118,7 +125,7 @@ class TestImagingCoreResampleAccuracy:
     def test_reduce_hamming(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (8, 8), 0xE1)
-            case = case.resize((4, 4), Image.HAMMING)
+            case = case.resize((4, 4), Image.Resampling.HAMMING)
             # fmt: off
             data = ("e1 da"
                     "da d3")
@@ -129,7 +136,7 @@ class TestImagingCoreResampleAccuracy:
     def test_reduce_bicubic(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (12, 12), 0xE1)
-            case = case.resize((6, 6), Image.BICUBIC)
+            case = case.resize((6, 6), Image.Resampling.BICUBIC)
             # fmt: off
             data = ("e1 e3 d4"
                     "e3 e5 d6"
@@ -141,7 +148,7 @@ class TestImagingCoreResampleAccuracy:
     def test_reduce_lanczos(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (16, 16), 0xE1)
-            case = case.resize((8, 8), Image.LANCZOS)
+            case = case.resize((8, 8), Image.Resampling.LANCZOS)
             # fmt: off
             data = ("e1 e0 e4 d7"
                     "e0 df e3 d6"
@@ -154,7 +161,7 @@ class TestImagingCoreResampleAccuracy:
     def test_enlarge_box(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (2, 2), 0xE1)
-            case = case.resize((4, 4), Image.BOX)
+            case = case.resize((4, 4), Image.Resampling.BOX)
             # fmt: off
             data = ("e1 e1"
                     "e1 e1")
@@ -165,7 +172,7 @@ class TestImagingCoreResampleAccuracy:
     def test_enlarge_bilinear(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (2, 2), 0xE1)
-            case = case.resize((4, 4), Image.BILINEAR)
+            case = case.resize((4, 4), Image.Resampling.BILINEAR)
             # fmt: off
             data = ("e1 b0"
                     "b0 98")
@@ -176,7 +183,7 @@ class TestImagingCoreResampleAccuracy:
     def test_enlarge_hamming(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (2, 2), 0xE1)
-            case = case.resize((4, 4), Image.HAMMING)
+            case = case.resize((4, 4), Image.Resampling.HAMMING)
             # fmt: off
             data = ("e1 d2"
                     "d2 c5")
@@ -187,7 +194,7 @@ class TestImagingCoreResampleAccuracy:
     def test_enlarge_bicubic(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (4, 4), 0xE1)
-            case = case.resize((8, 8), Image.BICUBIC)
+            case = case.resize((8, 8), Image.Resampling.BICUBIC)
             # fmt: off
             data = ("e1 e5 ee b9"
                     "e5 e9 f3 bc"
@@ -200,7 +207,7 @@ class TestImagingCoreResampleAccuracy:
     def test_enlarge_lanczos(self):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (6, 6), 0xE1)
-            case = case.resize((12, 12), Image.LANCZOS)
+            case = case.resize((12, 12), Image.Resampling.LANCZOS)
             data = (
                 "e1 e0 db ed f5 b8"
                 "e0 df da ec f3 b7"
@@ -213,15 +220,17 @@ class TestImagingCoreResampleAccuracy:
                 self.check_case(channel, self.make_sample(data, (12, 12)))
 
     def test_box_filter_correct_range(self):
-        im = Image.new("RGB", (8, 8), "#1688ff").resize((100, 100), Image.BOX)
+        im = Image.new("RGB", (8, 8), "#1688ff").resize(
+            (100, 100), Image.Resampling.BOX
+        )
         ref = Image.new("RGB", (100, 100), "#1688ff")
         assert_image_equal(im, ref)
 
 
-class CoreResampleConsistencyTest:
+class TestCoreResampleConsistency:
     def make_case(self, mode, fill):
         im = Image.new(mode, (512, 9), fill)
-        return im.resize((9, 512), Image.LANCZOS), im.load()[0, 0]
+        return im.resize((9, 512), Image.Resampling.LANCZOS), im.load()[0, 0]
 
     def run_case(self, case):
         channel, color = case
@@ -229,7 +238,7 @@ class CoreResampleConsistencyTest:
         for x in range(channel.size[0]):
             for y in range(channel.size[1]):
                 if px[x, y] != color:
-                    message = "{} != {} for pixel {}".format(px[x, y], color, (x, y))
+                    message = f"{px[x, y]} != {color} for pixel {(x, y)}"
                     assert px[x, y] == color, message
 
     def test_8u(self):
@@ -253,7 +262,7 @@ class CoreResampleConsistencyTest:
         self.run_case(self.make_case("F", 1.192093e-07))
 
 
-class CoreResampleAlphaCorrectTest:
+class TestCoreResampleAlphaCorrect:
     def make_levels_case(self, mode):
         i = Image.new(mode, (256, 16))
         px = i.load()
@@ -268,29 +277,28 @@ class CoreResampleAlphaCorrectTest:
         px = i.load()
         for y in range(i.size[1]):
             used_colors = {px[x, y][0] for x in range(i.size[0])}
-            assert 256 == len(
-                used_colors
-            ), "All colors should present in resized image. Only {} on {} line.".format(
-                len(used_colors), y
+            assert 256 == len(used_colors), (
+                "All colors should be present in resized image. "
+                f"Only {len(used_colors)} on {y} line."
             )
 
-    @pytest.mark.skip("Current implementation isn't precise enough")
+    @pytest.mark.xfail(reason="Current implementation isn't precise enough")
     def test_levels_rgba(self):
         case = self.make_levels_case("RGBA")
-        self.run_levels_case(case.resize((512, 32), Image.BOX))
-        self.run_levels_case(case.resize((512, 32), Image.BILINEAR))
-        self.run_levels_case(case.resize((512, 32), Image.HAMMING))
-        self.run_levels_case(case.resize((512, 32), Image.BICUBIC))
-        self.run_levels_case(case.resize((512, 32), Image.LANCZOS))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BOX))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BILINEAR))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.HAMMING))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BICUBIC))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.LANCZOS))
 
-    @pytest.mark.skip("Current implementation isn't precise enough")
+    @pytest.mark.xfail(reason="Current implementation isn't precise enough")
     def test_levels_la(self):
         case = self.make_levels_case("LA")
-        self.run_levels_case(case.resize((512, 32), Image.BOX))
-        self.run_levels_case(case.resize((512, 32), Image.BILINEAR))
-        self.run_levels_case(case.resize((512, 32), Image.HAMMING))
-        self.run_levels_case(case.resize((512, 32), Image.BICUBIC))
-        self.run_levels_case(case.resize((512, 32), Image.LANCZOS))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BOX))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BILINEAR))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.HAMMING))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BICUBIC))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.LANCZOS))
 
     def make_dirty_case(self, mode, clean_pixel, dirty_pixel):
         i = Image.new(mode, (64, 64), dirty_pixel)
@@ -307,29 +315,38 @@ class CoreResampleAlphaCorrectTest:
         for y in range(i.size[1]):
             for x in range(i.size[0]):
                 if px[x, y][-1] != 0 and px[x, y][:-1] != clean_pixel:
-                    message = "pixel at ({}, {}) is differ:\n{}\n{}".format(
-                        x, y, px[x, y], clean_pixel
+                    message = (
+                        f"pixel at ({x}, {y}) is different:\n"
+                        f"{px[x, y]}\n{clean_pixel}"
                     )
                     assert px[x, y][:3] == clean_pixel, message
 
     def test_dirty_pixels_rgba(self):
         case = self.make_dirty_case("RGBA", (255, 255, 0, 128), (0, 0, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BOX), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BILINEAR), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.HAMMING), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BICUBIC), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.LANCZOS), (255, 255, 0))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.BOX), (255, 255, 0))
+        self.run_dirty_case(
+            case.resize((20, 20), Image.Resampling.BILINEAR), (255, 255, 0)
+        )
+        self.run_dirty_case(
+            case.resize((20, 20), Image.Resampling.HAMMING), (255, 255, 0)
+        )
+        self.run_dirty_case(
+            case.resize((20, 20), Image.Resampling.BICUBIC), (255, 255, 0)
+        )
+        self.run_dirty_case(
+            case.resize((20, 20), Image.Resampling.LANCZOS), (255, 255, 0)
+        )
 
     def test_dirty_pixels_la(self):
         case = self.make_dirty_case("LA", (255, 128), (0, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BOX), (255,))
-        self.run_dirty_case(case.resize((20, 20), Image.BILINEAR), (255,))
-        self.run_dirty_case(case.resize((20, 20), Image.HAMMING), (255,))
-        self.run_dirty_case(case.resize((20, 20), Image.BICUBIC), (255,))
-        self.run_dirty_case(case.resize((20, 20), Image.LANCZOS), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.BOX), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.BILINEAR), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.HAMMING), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.BICUBIC), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.LANCZOS), (255,))
 
 
-class CoreResamplePassesTest:
+class TestCoreResamplePasses:
     @contextmanager
     def count(self, diff):
         count = Image.core.get_stats()["new_count"]
@@ -339,26 +356,26 @@ class CoreResamplePassesTest:
     def test_horizontal(self):
         im = hopper("L")
         with self.count(1):
-            im.resize((im.size[0] - 10, im.size[1]), Image.BILINEAR)
+            im.resize((im.size[0] - 10, im.size[1]), Image.Resampling.BILINEAR)
 
     def test_vertical(self):
         im = hopper("L")
         with self.count(1):
-            im.resize((im.size[0], im.size[1] - 10), Image.BILINEAR)
+            im.resize((im.size[0], im.size[1] - 10), Image.Resampling.BILINEAR)
 
     def test_both(self):
         im = hopper("L")
         with self.count(2):
-            im.resize((im.size[0] - 10, im.size[1] - 10), Image.BILINEAR)
+            im.resize((im.size[0] - 10, im.size[1] - 10), Image.Resampling.BILINEAR)
 
     def test_box_horizontal(self):
         im = hopper("L")
         box = (20, 0, im.size[0] - 20, im.size[1])
         with self.count(1):
             # the same size, but different box
-            with_box = im.resize(im.size, Image.BILINEAR, box)
+            with_box = im.resize(im.size, Image.Resampling.BILINEAR, box)
         with self.count(2):
-            cropped = im.crop(box).resize(im.size, Image.BILINEAR)
+            cropped = im.crop(box).resize(im.size, Image.Resampling.BILINEAR)
         assert_image_similar(with_box, cropped, 0.1)
 
     def test_box_vertical(self):
@@ -366,13 +383,13 @@ class CoreResamplePassesTest:
         box = (0, 20, im.size[0], im.size[1] - 20)
         with self.count(1):
             # the same size, but different box
-            with_box = im.resize(im.size, Image.BILINEAR, box)
+            with_box = im.resize(im.size, Image.Resampling.BILINEAR, box)
         with self.count(2):
-            cropped = im.crop(box).resize(im.size, Image.BILINEAR)
+            cropped = im.crop(box).resize(im.size, Image.Resampling.BILINEAR)
         assert_image_similar(with_box, cropped, 0.1)
 
 
-class CoreResampleCoefficientsTest:
+class TestCoreResampleCoefficients:
     def test_reduce(self):
         test_color = 254
 
@@ -381,7 +398,7 @@ class CoreResampleCoefficientsTest:
             draw = ImageDraw.Draw(i)
             draw.rectangle((0, 0, i.size[0] // 2 - 1, 0), test_color)
 
-            px = i.resize((5, i.size[1]), Image.BICUBIC).load()
+            px = i.resize((5, i.size[1]), Image.Resampling.BICUBIC).load()
             if px[2, 0] != test_color // 2:
                 assert test_color // 2 == px[2, 0]
 
@@ -389,7 +406,7 @@ class CoreResampleCoefficientsTest:
         # regression test for the wrong coefficients calculation
         # due to bug https://github.com/python-pillow/Pillow/issues/2161
         im = Image.new("RGBA", (1280, 1280), (0x20, 0x40, 0x60, 0xFF))
-        histogram = im.resize((256, 256), Image.BICUBIC).histogram()
+        histogram = im.resize((256, 256), Image.Resampling.BICUBIC).histogram()
 
         # first channel
         assert histogram[0x100 * 0 + 0x20] == 0x10000
@@ -401,16 +418,16 @@ class CoreResampleCoefficientsTest:
         assert histogram[0x100 * 3 + 0xFF] == 0x10000
 
 
-class CoreResampleBoxTest:
+class TestCoreResampleBox:
     def test_wrong_arguments(self):
         im = hopper()
         for resample in (
-            Image.NEAREST,
-            Image.BOX,
-            Image.BILINEAR,
-            Image.HAMMING,
-            Image.BICUBIC,
-            Image.LANCZOS,
+            Image.Resampling.NEAREST,
+            Image.Resampling.BOX,
+            Image.Resampling.BILINEAR,
+            Image.Resampling.HAMMING,
+            Image.Resampling.BICUBIC,
+            Image.Resampling.LANCZOS,
         ):
             im.resize((32, 32), resample, (0, 0, im.width, im.height))
             im.resize((32, 32), resample, (20, 20, im.width, im.height))
@@ -449,20 +466,26 @@ class CoreResampleBoxTest:
         for y0, y1 in split_range(dst_size[1], ytiles):
             for x0, x1 in split_range(dst_size[0], xtiles):
                 box = (x0 * scale[0], y0 * scale[1], x1 * scale[0], y1 * scale[1])
-                tile = im.resize((x1 - x0, y1 - y0), Image.BICUBIC, box)
+                tile = im.resize((x1 - x0, y1 - y0), Image.Resampling.BICUBIC, box)
                 tiled.paste(tile, (x0, y0))
         return tiled
 
+    @mark_if_feature_version(
+        pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
+    )
     def test_tiles(self):
         with Image.open("Tests/images/flower.jpg") as im:
             assert im.size == (480, 360)
             dst_size = (251, 188)
-            reference = im.resize(dst_size, Image.BICUBIC)
+            reference = im.resize(dst_size, Image.Resampling.BICUBIC)
 
             for tiles in [(1, 1), (3, 3), (9, 7), (100, 100)]:
                 tiled = self.resize_tiled(im, dst_size, *tiles)
                 assert_image_similar(reference, tiled, 0.01)
 
+    @mark_if_feature_version(
+        pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
+    )
     def test_subsample(self):
         # This test shows advantages of the subpixel resizing
         # after supersampling (e.g. during JPEG decoding).
@@ -470,12 +493,16 @@ class CoreResampleBoxTest:
             assert im.size == (480, 360)
             dst_size = (48, 36)
             # Reference is cropped image resized to destination
-            reference = im.crop((0, 0, 473, 353)).resize(dst_size, Image.BICUBIC)
-            # Image.BOX emulates supersampling (480 / 8 = 60, 360 / 8 = 45)
-            supersampled = im.resize((60, 45), Image.BOX)
+            reference = im.crop((0, 0, 473, 353)).resize(
+                dst_size, Image.Resampling.BICUBIC
+            )
+            # Image.Resampling.BOX emulates supersampling (480 / 8 = 60, 360 / 8 = 45)
+            supersampled = im.resize((60, 45), Image.Resampling.BOX)
 
-        with_box = supersampled.resize(dst_size, Image.BICUBIC, (0, 0, 59.125, 44.125))
-        without_box = supersampled.resize(dst_size, Image.BICUBIC)
+        with_box = supersampled.resize(
+            dst_size, Image.Resampling.BICUBIC, (0, 0, 59.125, 44.125)
+        )
+        without_box = supersampled.resize(dst_size, Image.Resampling.BICUBIC)
 
         # error with box should be much smaller than without
         assert_image_similar(reference, with_box, 6)
@@ -483,7 +510,7 @@ class CoreResampleBoxTest:
             assert_image_similar(reference, without_box, 5)
 
     def test_formats(self):
-        for resample in [Image.NEAREST, Image.BILINEAR]:
+        for resample in [Image.Resampling.NEAREST, Image.Resampling.BILINEAR]:
             for mode in ["RGB", "L", "RGBA", "LA", "I", ""]:
                 im = hopper(mode)
                 box = (20, 20, im.size[0] - 20, im.size[1] - 20)
@@ -501,9 +528,9 @@ class CoreResampleBoxTest:
             ((40, 50), (10, 0, 50, 50)),
             ((40, 50), (10, 20, 50, 70)),
         ]:
-            res = im.resize(size, Image.LANCZOS, box)
+            res = im.resize(size, Image.Resampling.LANCZOS, box)
             assert res.size == size
-            assert_image_equal(res, im.crop(box), ">>> {} {}".format(size, box))
+            assert_image_equal(res, im.crop(box), f">>> {size} {box}")
 
     def test_no_passthrough(self):
         # When resize is required
@@ -515,19 +542,17 @@ class CoreResampleBoxTest:
             ((40, 50), (10.4, 0.4, 50.4, 50.4)),
             ((40, 50), (10.4, 20.4, 50.4, 70.4)),
         ]:
-            res = im.resize(size, Image.LANCZOS, box)
+            res = im.resize(size, Image.Resampling.LANCZOS, box)
             assert res.size == size
             with pytest.raises(AssertionError, match=r"difference \d"):
                 # check that the difference at least that much
-                assert_image_similar(
-                    res, im.crop(box), 20, ">>> {} {}".format(size, box)
-                )
+                assert_image_similar(res, im.crop(box), 20, f">>> {size} {box}")
 
     def test_skip_horizontal(self):
         # Can skip resize for one dimension
         im = hopper()
 
-        for flt in [Image.NEAREST, Image.BICUBIC]:
+        for flt in [Image.Resampling.NEAREST, Image.Resampling.BICUBIC]:
             for size, box in [
                 ((40, 50), (0, 0, 40, 90)),
                 ((40, 50), (0, 20, 40, 90)),
@@ -541,14 +566,14 @@ class CoreResampleBoxTest:
                     res,
                     im.crop(box).resize(size, flt),
                     0.4,
-                    ">>> {} {} {}".format(size, box, flt),
+                    f">>> {size} {box} {flt}",
                 )
 
     def test_skip_vertical(self):
         # Can skip resize for one dimension
         im = hopper()
 
-        for flt in [Image.NEAREST, Image.BICUBIC]:
+        for flt in [Image.Resampling.NEAREST, Image.Resampling.BICUBIC]:
             for size, box in [
                 ((40, 50), (0, 0, 90, 50)),
                 ((40, 50), (20, 0, 90, 50)),
@@ -562,5 +587,5 @@ class CoreResampleBoxTest:
                     res,
                     im.crop(box).resize(size, flt),
                     0.4,
-                    ">>> {} {} {}".format(size, box, flt),
+                    f">>> {size} {box} {flt}",
                 )
